@@ -1,6 +1,4 @@
 from pyramid.config import Configurator
-from sqlalchemy import engine_from_config
-from skosprovider_sqlalchemy.models import Base as SkosBase
 from atramhasis.renderers import json_renderer_verbose
 from .models import Base
 
@@ -12,7 +10,12 @@ def includeme(config):
     config.add_static_view('static', 'static', cache_max_age=3600)
     config.add_renderer('csv', 'atramhasis.renderers.CSVRenderer')
     config.add_renderer('skosrenderer_verbose', json_renderer_verbose)
+    # Rewrite urls with trailing slash
+    config.include('pyramid_rewrite')
+    config.add_rewrite_rule(r'/(?P<path>.*)/', r'/%(path)s')
     config.add_route('home', '/')
+    config.add_route('conceptschemes', pattern='/conceptschemes', accept='text/html', request_method="GET")
+    config.add_route('conceptscheme', pattern='/conceptschemes/{scheme_id}', accept='text/html', request_method="GET")
     config.add_route('concept', pattern='/conceptschemes/{scheme_id}/c/{c_id}', accept='text/html',
                      request_method="GET")
     config.add_route('search_result', pattern='/conceptschemes/{scheme_id}/c', accept='text/html')
@@ -27,8 +30,15 @@ def includeme(config):
                      request_method="PUT")
     config.add_route('atramhasis.delete_concept', pattern='/conceptschemes/{scheme_id}/c/{c_id}', accept='application/json',
                      request_method="DELETE")
+    config.add_route('atramhasis.list_languages', pattern='/languages', accept='application/json',
+                     request_method="GET")
+    config.add_route('atramhasis.get_language', pattern='/languages/{l_id}', accept='application/json',
+                     request_method="GET")
+    config.add_route('atramhasis.edit_language', pattern='/languages/{l_id}', accept='application/json',
+                     request_method="PUT")
+    config.add_route('atramhasis.delete_language', pattern='/languages/{l_id}', accept='application/json',
+                     request_method="DELETE")
     config.add_route('locale', '/locale')
-
     config.add_route('labeltypes', '/labeltypes', accept='application/json', request_method="GET")
     config.add_route('notetypes', '/notetypes', accept='application/json', request_method="GET")
 
@@ -46,9 +56,6 @@ def includeme(config):
 def main(global_config, **settings):
     """ This function returns a Pyramid WSGI application.
     """
-    engine = engine_from_config(settings, 'sqlalchemy.')
-    Base.metadata.bind = engine
-    SkosBase.metadata.bind = engine
     config = Configurator(settings=settings)
     includeme(config)
 
@@ -57,6 +64,8 @@ def main(global_config, **settings):
     config.include('atramhasis:db')
 
     # if standalone include skos sample data
-    config.include('.skos')
+    test_mode = settings.get('atramhasis.test_mode')
+    if not test_mode == 'true':   # pragma: no cover
+        config.include('.skos')
 
     return config.make_wsgi_app()
