@@ -3,7 +3,7 @@
 Module containing mapping functions used by Atramhasis.
 '''
 
-from skosprovider_sqlalchemy.models import Label, Note, Concept, Thing, Collection
+from skosprovider_sqlalchemy.models import Label, Note, Concept, Thing, Collection, Match, MatchType
 from sqlalchemy.orm.exc import NoResultFound
 
 
@@ -36,51 +36,89 @@ def map_concept(concept, concept_json, db_session):
 
         concept.member_of.clear()
         member_of = concept_json.get('member_of', [])
-        for memberof_id in member_of:
+        for memberof in member_of:
             try:
                 memberof_collection = db_session.query(Collection)\
-                    .filter_by(concept_id=memberof_id, conceptscheme_id=concept.conceptscheme_id).one()
+                    .filter_by(concept_id=memberof['id'], conceptscheme_id=concept.conceptscheme_id).one()
             except NoResultFound:
-                memberof_collection = Collection(concept_id=memberof_id, conceptscheme_id=concept.conceptscheme_id)
+                memberof_collection = Collection(concept_id=memberof['id'], conceptscheme_id=concept.conceptscheme_id)
             concept.member_of.add(memberof_collection)
 
         if concept.type == 'concept':
             concept.related_concepts.clear()
             related = concept_json.get('related', [])
-            for related_id in related:
+            for related in related:
                 try:
-                    related_concept = db_session.query(Concept).filter_by(concept_id=related_id,
+                    related_concept = db_session.query(Concept).filter_by(concept_id=related['id'],
                                                                           conceptscheme_id=concept.conceptscheme_id).one()
                 except NoResultFound:
-                    related_concept = Concept(concept_id=related_id, conceptscheme_id=concept.conceptscheme_id)
+                    related_concept = Concept(concept_id=related['id'], conceptscheme_id=concept.conceptscheme_id)
                 concept.related_concepts.add(related_concept)
             concept.narrower_concepts.clear()
 
             concept.broader_concepts.clear()
             broader = concept_json.get('broader', [])
-            for broader_id in broader:
+            for broader in broader:
                 try:
-                    broader_concept = db_session.query(Concept).filter_by(concept_id=broader_id,
+                    broader_concept = db_session.query(Concept).filter_by(concept_id=broader['id'],
                                                                           conceptscheme_id=concept.conceptscheme_id).one()
                 except NoResultFound:
-                    broader_concept = Concept(concept_id=broader_id, conceptscheme_id=concept.conceptscheme_id)
+                    broader_concept = Concept(concept_id=broader['id'], conceptscheme_id=concept.conceptscheme_id)
                 concept.broader_concepts.add(broader_concept)
             narrower = concept_json.get('narrower', [])
-            for narrower_id in narrower:
+            for narrower in narrower:
                 try:
-                    narrower_concept = db_session.query(Concept).filter_by(concept_id=narrower_id,
+                    narrower_concept = db_session.query(Concept).filter_by(concept_id=narrower['id'],
                                                                            conceptscheme_id=concept.conceptscheme_id).one()
                 except NoResultFound:
-                    narrower_concept = Concept(concept_id=narrower_id, conceptscheme_id=concept.conceptscheme_id)
+                    narrower_concept = Concept(concept_id=narrower['id'], conceptscheme_id=concept.conceptscheme_id)
                 concept.narrower_concepts.add(narrower_concept)
+
+            matches = []
+            matchdict = concept_json.get('matches', {})
+            for type in matchdict:
+                db_type = type + "Match"
+                matchtype = db_session.query(MatchType).filter_by(name=db_type).one()
+                for uri in matchdict[type]:
+                    concept_id = concept_json.get('id', -1)
+                    try:
+                        match = db_session.query(Match).filter_by(uri=uri, matchtype_id=matchtype.name, concept_id=concept_id).one()
+                    except NoResultFound:
+                        match = Match()
+                        match.matchtype = matchtype
+                        match.uri = uri
+                    matches.append(match)
+            concept.matches = matches
+
+            concept.narrower_collections.clear()
+            narrower_collections = concept_json.get('subordinate_arrays', [])
+            for narrower in narrower_collections:
+                try:
+                    narrower_collection = db_session.query(Collection)\
+                        .filter_by(concept_id=narrower['id'], conceptscheme_id=concept.conceptscheme_id).one()
+                except NoResultFound:
+                    narrower_collection = Collection(concept_id=narrower['id'], conceptscheme_id=concept.conceptscheme_id)
+                concept.narrower_collections.add(narrower_collection)
+
         if concept.type == 'collection':
             concept.members.clear()
             members = concept_json.get('members', [])
-            for member_id in members:
+            for member in members:
                 try:
-                    member_concept = db_session.query(Thing).filter_by(concept_id=member_id,
+                    member_concept = db_session.query(Thing).filter_by(concept_id=member['id'],
                                                                            conceptscheme_id=concept.conceptscheme_id).one()
                 except NoResultFound:
-                    member_concept = Concept(concept_id=member_id, conceptscheme_id=concept.conceptscheme_id)
+                    member_concept = Concept(concept_id=member['id'], conceptscheme_id=concept.conceptscheme_id)
                 concept.members.add(member_concept)
+
+            concept.broader_concepts.clear()
+            broader_concepts = concept_json.get('superordinates', [])
+            for broader in broader_concepts:
+                try:
+                    broader_concept = db_session.query(Concept)\
+                        .filter_by(concept_id=broader['id'], conceptscheme_id=concept.conceptscheme_id).one()
+                except NoResultFound:
+                    broader_concept = Concept(concept_id=broader['id'], conceptscheme_id=concept.conceptscheme_id)
+                concept.broader_concepts.add(broader_concept)
+
     return concept
