@@ -37,6 +37,7 @@ define([
                 labels: null,
                 tempLabels: null,//this variable is used to recover the labels if user delete a label and then press on the cancel button
                 EditLabelButton: null,
+                languageStore: null,
 
                 buildRendering: function () {
                     this.inherited(arguments);
@@ -54,6 +55,7 @@ define([
                             var dlg = self._createDialog();
                             if (self.labels) {
                                 self._setGrid(self.labels);
+                                self.tempLabels = lang.clone(self.labels);
                                // self._setLanguageComboBox(self.labels);
                             }
                             dlg.show();
@@ -96,20 +98,15 @@ define([
                             style: { width: '120px' }
 
                         });
-                    var languages = this._getLanguages();
-                    self.prefLanguage = languages;
-                    var langStoreComboBox = new Select
-                    (
-                        {
-                            id: "langStoreComboBox",
-                            name: "langStoreComboBox",
-                            title: "Language:",
-                            placeHolder: 'Select a language',
-                            options: languages,
-                            style: { width: '80px' }
 
-                        }
-                    );
+                    var langStoreComboBox = new Select({
+                        id: "langStoreComboBox",
+                        name: "langStoreComboBox",
+                        title: "Language:",
+                        store: this.languageStore,
+                        style: { width: '80px' },
+                        labelAttr: "name"
+                    });
 
                     var addLabelButtonToTable = new Button
                     (
@@ -172,6 +169,7 @@ define([
                         dlg.hide();
                     };
                     cancelBtn.onClick = function () {
+                        self.labels = lang.clone(self.tempLabels);
                         dlg.hide();
                     };
 
@@ -183,7 +181,6 @@ define([
                     );
 
                     on(dlg, "hide", function () {
-                        self.labels = lang.clone(self.tempLabels);
                         titleLabel.destroy();
                         labelTypeComboBox.destroy();
                         langStoreComboBox.destroy();
@@ -205,17 +202,6 @@ define([
                     return Type;
                 },
 
-                _getLanguages: function () {
-                    var languages = [
-                        {label: "NL", value: "nl", disabled: false},
-                        {label: "FR", value: "fr", disabled: false},
-                        {label: "EN", value: "en", disabled: false}
-
-                    ];
-
-                    return languages;
-
-                },
                 _createGrid: function (gridDiv) {
                     var self = this;
                     var columns;
@@ -255,11 +241,11 @@ define([
                 },
                 _createNodeList: function (labels) {
                     var mapLabel = this.prefLabelList.mapLabelsForList(labels, "prefLabel");
-                    this.prefLabelList.buidList(mapLabel, "Preferred labels", false);
+                    this.prefLabelList.buildList(mapLabel, "Preferred labels", false);
                     mapLabel = this.altLabelList.mapLabelsForList(labels, "altLabel");
-                    this.altLabelList.buidList(mapLabel, "Alternate labels", false);
+                    this.altLabelList.buildList(mapLabel, "Alternate labels", false);
                     mapLabel = this.hiddenLabelList.mapLabelsForList(labels, "hiddenLabel");
-                    this.hiddenLabelList.buidList(mapLabel, "Hidden labels", false);
+                    this.hiddenLabelList.buildList(mapLabel, "Hidden labels", false);
                 },
                 _setGrid: function (labels) {
                     var gridStore = new Memory({
